/*******************************************************************************
 * (c) Copyright 2016 Microsemi Corporation.  All rights reserved.
 *
 */

//#include <stdio.h>

#include "hw_platform.h"
#include "drivers/CoreGPIO/core_gpio.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "drivers/CoreSPI/core_spi.h"
#include "hal/CortexM1/cortex_nvic.h"
#include "./CMSIS/cortexm1_cfg.h"
#include "./CMSIS/system_cortexm1_cfg.h"
#include "./hal/hal.h"

#include "uart_terminal.h"
#include "freq_synths.h"
#include "ZL30265_image.h"
#include "ZL30722_image.h"


/******************************************************************************
 * GPIO instance data. UART instance data. Base addresses
 *****************************************************************************/
gpio_instance_t g_gpio;

/* for blink counter */
#define COUNTER_MAX					(SystemCoreClock / 512)


/* function prototypes ****************************************************** */
void init(
		UART_instance_t * g_uart0,
		spi_instance_t	* g_spi0,
		spi_instance_t	* g_spi1,
		gpio_instance_t * g_gpio
);
void update_LEDs(void);
void LED_4(int state);
void print_menu(UART_instance_t * g_uart);
void start_counters(void);
void read_counters(UART_instance_t * g_uart);


/* Semaphores for event handling ******************************************** */
int sema_gpio = 0;
int sema_cntr = 0;

/* interrupt handler ******************************************************** */
__attribute__((__interrupt__)) void IRQ1_IRQHandler(void)
{
	sema_gpio = 1;
	GPIO_clear_all_irq_sources(&g_gpio, GPIO_get_irq_sources(&g_gpio));
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_GPIO);
}
__attribute__((__interrupt__)) void IRQ2_IRQHandler(void)
{
	sema_cntr = 1;
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_CNTR);
}
/*-------------------------------------------------------------------------*//**
 * main() function.
 */
int main()
{
 	uint8_t  key;
	uint32_t counter = 0;

	UART_instance_t g_uart0;
	spi_instance_t	g_spi0, g_spi1;

	freq_synths_t g_ZL30265 = {
			.g_spi              = &g_spi0,
			.spi_slave          = SPI_SLAVE_0,
			.eeprom_select_byte = 0x80,
			.eeprom_data        = ZL30265_image,
			.eeprom_length      = ZL30265_IMAGE_SIZE
	};
	freq_synths_t g_ZL30722 = {
			.g_spi              = &g_spi1,
			.spi_slave          = SPI_SLAVE_0,
			.eeprom_select_byte = 0x01,
			.eeprom_data        = ZL30722_image,
			.eeprom_length      = ZL30722_IMAGE_SIZE
	};

	init(&g_uart0, &g_spi0, &g_spi1, &g_gpio);
	print_menu(&g_uart0);

	while(1) {

    	 if(UART_get_rx(&g_uart0, &key, 1)) {
    		 switch(key) {
    		 case 'c': freq_synths_ctrl(&g_uart0, &g_ZL30265); break;
    		 case 's': freq_synths_read_reg(&g_uart0, &g_ZL30265, 0x30, 0x59, "Status"); break;
    		 case 'g': freq_synths_read_reg(&g_uart0, &g_ZL30265, 0x00, 0x11, "Global"); break;
    		 case 'a': freq_synths_read_reg(&g_uart0, &g_ZL30265, 0x100, 0x1CE, "APLL"); break;
    		 case 'o': freq_synths_read_reg(&g_uart0, &g_ZL30265, 0x200, 0x29A, "Output clock"); break;
    		 case 'i': freq_synths_read_reg(&g_uart0, &g_ZL30265, 0x300, 0x305, "Input clock"); break;
    		 case 'r': freq_synths_read_eeprom(&g_uart0, &g_ZL30265); break;
    		 case 'w': freq_synths_eeprom_init(&g_uart0, &g_ZL30265); break;
    		 case '0': freq_synths_eeprom_init(&g_uart0, &g_ZL30722); break;
    		 case '1': freq_synths_read_eeprom(&g_uart0, &g_ZL30722); break;
    		 case 'm': start_counters(); LED_4(1); break;
    		 default: break;
    		 }
			 print_menu(&g_uart0);
    	 }


    	 if(sema_gpio) {
    		 counter = 0;
    		 sema_gpio = 0;
    	 }

    	 if(sema_cntr) {
    		 sema_cntr = 0;
    		 LED_4(0);
    		 read_counters(&g_uart0);
    	 }

    	 if(counter-- ==  0) {
    		 counter = COUNTER_MAX;
    		 update_LEDs();
    	 }
     }

     return 0;
}

void init(
		UART_instance_t * g_uart0,
		spi_instance_t	* g_spi0,
		spi_instance_t	* g_spi1,
		gpio_instance_t * g_gpio
)
{
	SystemCoreClockUpdate();

	UART_init(
			g_uart0,
			UART0_BASE_ADDRESS,
			BAUD_VALUE_115200,
			(DATA_8_BITS | NO_PARITY)
	);

	/* SPI init & config */
	SPI_init(g_spi0, SPI0_BASE_ADDRESS, 32);
	SPI_configure_master_mode(g_spi0);

	SPI_init(g_spi1, SPI1_BASE_ADDRESS, 32);
	SPI_configure_master_mode(g_spi1);

    /**************************************************************************
     * Initialize the CoreGPIO driver with the base address of the CoreGPIO
     * instance to use and the initial state of the outputs.
     *************************************************************************/
	/* OUTPUTS */
	GPIO_init(g_gpio, COREGPIO_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	/* INPUTS */
	GPIO_enable_irq(g_gpio, GPIO_0);
	GPIO_enable_irq(g_gpio, GPIO_1);
	GPIO_enable_irq(g_gpio, GPIO_2);
	GPIO_enable_irq(g_gpio, GPIO_3);

	HW_set_32bit_reg(NVIC_ENABLE_REG, (NVIC_IRQ_GPIO | NVIC_IRQ_CNTR));

}

void update_LEDs(void)
{
	static uint8_t temp = 1;

	temp = temp^1;
	GPIO_set_output(&g_gpio, GPIO_4 ,temp);
	GPIO_set_output(&g_gpio, GPIO_5 ,temp);
	GPIO_set_output(&g_gpio, GPIO_6 ,temp);
}

void LED_4(int state)
{
	GPIO_set_output(&g_gpio, GPIO_7, state);
	return;
}

void print_menu(UART_instance_t * g_uart)
{
	const uint8_t menu[] = {
			"\n\r"
			"\tc control (interactive register write)\n\r"
			"\ts read status registers\n\r"
			"\tg read global registers\n\r"
			"\ta read APPL registers\n\r"
			"\to read output registers\n\r"
			"\ti read input registers\n\r"
			"\tr read ZL30265 eeprom\n\r"
			"\tw write ZL30265 eeprom\n\r"
			"\t0 write ZL30722 eeprom\n\r"
			"\t1 read ZL30722 eeprom\n\r"
			"\tm measure counters\n\r"
			"\n\r"
	};
	UART_polled_tx_string(g_uart, menu);

	return;
}

void start_counters(void)
{
	const uint32_t cntr_start_bit = 0x00000001UL;
	uint32_t * pCntrRegs = (uint32_t *)CNTR_BASE_ADDRESS;

	pCntrRegs[0] = cntr_start_bit;

	return;
}

void read_counters(UART_instance_t * g_uart)
{
	uint32_t * pCntrRegs = (uint32_t *)CNTR_BASE_ADDRESS;
	int i;

	pCntrRegs[0] = 0;

	UART_polled_tx_string(g_uart, "read_counters:\n\r");

	for(i = 0; i < 10; i++) {
		UART_polled_tx_string(g_uart, htoa(pCntrRegs[i]));
		UART_polled_tx_string(g_uart, "\t");
		UART_polled_tx_string(g_uart, itoa(pCntrRegs[i]));
		UART_polled_tx_string(g_uart, "\n\r");

	}

	UART_polled_tx_string(g_uart, "\n\r");

	return;
}

void delay(uint32_t tms)
{
	int i;
	uint32_t delayInMilliSeconds = (SYS_M1_CLK_FREQ * tms) / 1000;

	for(i = 0; i < delayInMilliSeconds; i++)
		asm("NOP\n");

	return;
}

